import { getChartTrellis, getEntityHierarchy, getPropertyCaption, getPropertyHierarchy, mergeOptions } from '@metad/ocap-core';
import { chunk, groupBy, isArray } from 'lodash-es';
import { dataZoom } from './data-zoom';
import { isCoordinateCalendar, isCoordinateCartesian2d, isCoordinatePolar, isCoordinateSingleAxis } from './types';
import { pickEChartsGlobalOptions } from './utils';
export function gatherCoordinates(coordinates, type, options) {
    const echartsOptions = {
        dataset: [],
        title: [],
        grid: [],
        series: [],
        visualMap: [],
        tooltip: [],
        legend: [],
        dataZoom: dataZoom(options)
    };
    // console.log(coordinates)
    coordinates.forEach((coordinate, gridIndex) => {
        if (coordinate.title) {
            echartsOptions.title.push(coordinate.title);
        }
        echartsOptions.grid.push(coordinate.grid);
        coordinate.datasets.forEach(({ dataset, series }) => {
            series.forEach((series) => {
                if (isCoordinatePolar(coordinate)) {
                    echartsOptions.series.push({
                        ...series,
                        angleAxis: (echartsOptions.angleAxis?.length ?? 0) + (series.angleAxisIndex ?? 0),
                        radiusAxis: (echartsOptions.radiusAxis?.length ?? 0) + (series.radiusAxis ?? 0),
                        datasetIndex: echartsOptions.dataset.length + (series.datasetIndex ?? 0),
                        type: series.type ?? type,
                        coordinateSystem: 'polar'
                    });
                }
                else {
                    echartsOptions.series.push({
                        ...series,
                        xAxisIndex: (echartsOptions.xAxis?.length ?? 0) + (series.xAxisIndex ?? 0),
                        yAxisIndex: (echartsOptions.yAxis?.length ?? 0) + (series.yAxisIndex ?? 0),
                        datasetIndex: echartsOptions.dataset.length + (series.datasetIndex ?? 0),
                        type: series.type ?? type
                    });
                }
            });
            if (isArray(dataset)) {
                echartsOptions.dataset.push(...dataset);
            }
            else {
                echartsOptions.dataset.push({
                    ...dataset
                });
            }
            // if (type !== 'pie' && isCoordinateCartesian2d(coordinate)) {
            //   echartsOptions.xAxis = echartsOptions.xAxis ?? []
            //   echartsOptions.xAxis.push(
            //     ...(coordinate.xAxis?.map((xAxis) => ({
            //       ...xAxis,
            //       gridIndex
            //     })) ?? [])
            //   )
            //   echartsOptions.yAxis = echartsOptions.yAxis ?? []
            //   echartsOptions.yAxis.push(
            //     ...(coordinate.yAxis?.map((yAxis) => ({
            //       ...yAxis,
            //       gridIndex
            //     })) ?? [])
            //   )
            // }
            // if (isCoordinatePolar(coordinate)) {
            //   // polar
            //   echartsOptions.polar = echartsOptions.polar ?? []
            //   echartsOptions.polar.push(coordinate.polar ?? {})
            //   // radiusAxis
            //   echartsOptions.radiusAxis = echartsOptions.radiusAxis ?? []
            //   echartsOptions.radiusAxis.push({
            //     ...(coordinate.radiusAxis?.[0] ?? {}),
            //     polarIndex: gridIndex
            //   })
            //   // angleAxis
            //   echartsOptions.angleAxis = echartsOptions.angleAxis ?? []
            //   echartsOptions.angleAxis.push({
            //     ...(coordinate.angleAxis?.[0] ?? {}),
            //     polarIndex: gridIndex
            //   })
            // }
        });
        if (type !== 'pie' && isCoordinateCartesian2d(coordinate)) {
            echartsOptions.xAxis = echartsOptions.xAxis ?? [];
            echartsOptions.xAxis.push(...(coordinate.xAxis?.map((xAxis) => ({
                ...xAxis,
                gridIndex
            })) ?? []));
            echartsOptions.yAxis = echartsOptions.yAxis ?? [];
            echartsOptions.yAxis.push(...(coordinate.yAxis?.map((yAxis) => ({
                ...yAxis,
                gridIndex
            })) ?? []));
        }
        if (isCoordinatePolar(coordinate)) {
            // polar
            echartsOptions.polar = echartsOptions.polar ?? [];
            echartsOptions.polar.push(coordinate.polar ?? {});
            // radiusAxis
            echartsOptions.radiusAxis = echartsOptions.radiusAxis ?? [];
            echartsOptions.radiusAxis.push({
                ...(coordinate.radiusAxis?.[0] ?? {}),
                polarIndex: gridIndex
            });
            // angleAxis
            echartsOptions.angleAxis = echartsOptions.angleAxis ?? [];
            echartsOptions.angleAxis.push({
                ...(coordinate.angleAxis?.[0] ?? {}),
                polarIndex: gridIndex
            });
        }
        coordinate.visualMap && echartsOptions.visualMap.push(...coordinate.visualMap);
        coordinate.tooltip && echartsOptions.tooltip.push(...coordinate.tooltip);
        coordinate.legend && echartsOptions.legend.push(...coordinate.legend);
        if (isCoordinateSingleAxis(coordinate)) {
            echartsOptions.singleAxis = echartsOptions.singleAxis ?? [];
            echartsOptions.singleAxis.push(...coordinate.singleAxis);
        }
        if (isCoordinateCalendar(coordinate)) {
            echartsOptions.calendar = echartsOptions.calendar ?? [];
            echartsOptions.calendar.push(coordinate.calendar);
        }
    });
    mergeOptions(echartsOptions, pickEChartsGlobalOptions(options));
    return echartsOptions;
}
/**
 * get coordinates by trellis or single coordinate
 *
 * @param data
 * @param chartAnnotation
 * @param entityType
 * @param settings
 * @param options
 * @param coordinateType
 * @param cartesianCoordinate
 * @returns
 */
export function coordinates(data, chartAnnotation, entityType, settings, options, coordinateType, cartesianCoordinate) {
    const trellis = getChartTrellis(chartAnnotation);
    if (trellis) {
        const trellisName = getPropertyHierarchy(trellis);
        const trellisCaption = getPropertyCaption(getEntityHierarchy(entityType, trellis));
        const trellisResults = groupBy(data.data, trellisName);
        const coordinates = Object.keys(trellisResults).map((trellisKey) => {
            const dimensions = [...chartAnnotation.dimensions];
            const index = dimensions.indexOf(trellis);
            dimensions.splice(index, 1);
            const coordinate = cartesianCoordinate(trellisResults[trellisKey], {
                ...chartAnnotation,
                dimensions
            }, entityType, settings, options, data.schema);
            coordinate.name = trellisResults[trellisKey][0]?.[trellisCaption] || trellisKey;
            coordinate.datasets = coordinate.datasets.map((dataset) => ({
                ...dataset,
                series: dataset.series.map((series) => ({
                    ...series,
                    id: `${trellisKey}-${series.id}`
                }))
            }));
            return coordinate;
        });
        const trellisHorizontal = settings?.trellisHorizontal ?? 2;
        const trellisVertical = Math.ceil(coordinates.length / trellisHorizontal);
        const coordinatesGroups = chunk(coordinates, trellisHorizontal);
        const coordinateResults = [];
        coordinatesGroups.forEach((group, v) => {
            group.forEach((coordinate, h) => {
                // Trellis title
                const title = {
                    text: coordinate.name,
                    textStyle: {
                        fontSize: '1rem'
                    },
                    left: (h + 0.4) * (100 / trellisHorizontal) + '%',
                    top: v * (100 / trellisVertical) + '%',
                };
                if (coordinateType === 'grid') {
                    coordinateResults.push({
                        ...coordinate,
                        title,
                        grid: {
                            ...coordinate.grid,
                            left: (h + 0.1) * (100 / trellisHorizontal) + '%',
                            top: (v + 0.2) * (100 / trellisVertical) + '%',
                            width: (100 / trellisHorizontal) * 0.8 + '%',
                            height: (100 / trellisVertical) * 0.7 + '%'
                        },
                    });
                }
                else if (coordinateType === 'pie') {
                    const minRadius = Math.max(100 / trellisHorizontal, 100 / trellisVertical);
                    const radius = (chartAnnotation.chartType.variant === 'Doughnut' ||
                        chartAnnotation.chartType.variant === 'Nightingale') ?
                        [minRadius * 0.4 + '%', minRadius * 0.7 + '%'] :
                        minRadius * 0.7 + '%';
                    coordinateResults.push({
                        ...coordinate,
                        title,
                        datasets: coordinate.datasets.map((dataset) => ({
                            ...dataset,
                            series: dataset.series.map((series) => ({
                                ...series,
                                radius,
                                center: [
                                    (h + 0.5) * (100 / trellisHorizontal) + '%',
                                    (v + 0.5) * (100 / trellisVertical) + '%',
                                ]
                            }))
                        }))
                    });
                }
            });
        });
        return coordinateResults;
    }
    return [cartesianCoordinate(data.data, chartAnnotation, entityType, settings, options, data.schema)];
}
//# sourceMappingURL=coordinates.js.map