import { ChartDimensionRoleType, ChartOrient, getChartCategory, getChartCategory2, getEntityHierarchy, getEntityLevel, getEntityProperty, getPropertyCaption, getPropertyHierarchy, mergeOptions, omitBlank } from '@metad/ocap-core';
import { HeatmapChart } from 'echarts/charts';
import { use } from 'echarts/core';
import { minBy, maxBy, isEmpty, cloneDeep, isNil } from 'lodash-es';
import format from 'date-fns/format';
import parse from 'date-fns/parse';
import { getCoordinateSystem, measuresToSeriesComponents, serializeSeriesComponent } from './cartesian';
import { coordinates, gatherCoordinates } from './coordinates';
import { getEChartsTooltip } from './tooltip';
import { AxisEnum } from './types';
import { getColorLightnessVisualMap, getMeasurePaletteVisualMap } from './visualMap';
use([HeatmapChart]);
export function heatmap(data, chartAnnotation, entityType, settings, options) {
    // 智能补全信息
    const index = chartAnnotation.dimensions.findIndex((item) => item.role === ChartDimensionRoleType.Category2);
    if (index === -1) {
        chartAnnotation = cloneDeep(chartAnnotation);
        const dimension = chartAnnotation.dimensions.slice(1).find((item) => !item.role);
        if (dimension) {
            dimension.role = ChartDimensionRoleType.Category2;
        }
    }
    const type = 'heatmap';
    const coordinateDatas = coordinates(data, chartAnnotation, entityType, settings, options, 'grid', chartAnnotation.chartType.variant === 'calendar' ? calendarCoordinate : cartesianCoordinate);
    return gatherCoordinates(coordinateDatas, type, options);
}
/**
 * 单个笛卡尔坐标系, 对应一个 grid 内的组件
 *
 * @param data 数据数组
 * @param chartAnnotation 图形注解
 * @param entityType 实体类型
 * @param settings 图形设置
 * @param options 图形属性
 * @returns
 */
export function cartesianCoordinate(data, chartAnnotation, entityType, settings, options) {
    const category = getChartCategory(chartAnnotation);
    const categoryProperty = getEntityProperty(entityType, category);
    const category2 = getChartCategory2(chartAnnotation);
    let datasets = [];
    const seriesComponents = measuresToSeriesComponents(chartAnnotation.measures, data, entityType, settings);
    data = [...data];
    // TODO 是不是应该提到所有图形之前 ?
    // Fill up measure null value
    if (data[0]) {
        data[0] = { ...data[0] };
        chartAnnotation.measures.forEach((measure) => {
            if (isNil(data[0][measure.measure])) {
                data[0][measure.measure] = '-';
            }
        });
    }
    datasets = [
        {
            dataset: {
                source: data
            },
            seriesComponents,
            tooltip: getEChartsTooltip(options?.tooltip, categoryProperty, chartAnnotation.measures.map((measure) => ({
                measure,
                property: getEntityProperty(entityType, measure)
            })), seriesComponents, settings.locale)
        }
    ];
    const { categoryAxis, valueAxis } = getCoordinateSystem(chartAnnotation, entityType, data, options, settings.locale);
    const gridOptions = {
        type: 'Cartesian2d',
        name: '',
        grid: mergeOptions({
            top: 0,
            right: 0,
            bottom: 0,
            left: 0,
            containLabel: true
        }, options?.grid),
        [categoryAxis.orient]: categoryAxis.axis,
        [valueAxis.orient]: valueAxis.axis,
        visualMap: [],
        datasets: [],
        tooltip: [],
        legend: [],
    };
    datasets.forEach(({ dataset, seriesComponents, tooltip }) => {
        gridOptions.datasets.push({
            dataset,
            series: seriesComponents.map((seriesComponent) => {
                const { series, visualMaps } = serializeSeriesComponent(dataset, seriesComponent, entityType, valueAxis, settings, options);
                gridOptions.visualMap.push(...visualMaps);
                if (!isNil(seriesComponent.valueAxisIndex)) {
                    series[valueAxis.orient + 'Index'] = seriesComponent.valueAxisIndex;
                }
                /**
                 * 堆积系列情况下不适用 encode, 使用的应该是 seriesLayoutBy: "row" name: "[Canada]" 与 dataset 中的行数据进行的对应
                 */
                if (isNil(series.seriesLayoutBy)) {
                    series.encode = {
                        [AxisEnum[categoryAxis.orient]]: getPropertyHierarchy(category),
                        [AxisEnum[valueAxis.orient]]: category2 ? getPropertyHierarchy(category2) : seriesComponent.measure,
                        value: chartAnnotation.measures[0].measure,
                        tooltip: seriesComponent.tooltip
                    };
                }
                series.tooltip = tooltip;
                mergeOptions(series, options?.seriesStyle);
                return series;
            })
        });
        // Default VisualMap
        if (isEmpty(gridOptions.visualMap)) {
            const measure = chartAnnotation.measures[0];
            const lightnessProperty = getEntityProperty(entityType, measure);
            gridOptions.visualMap.push(getColorLightnessVisualMap(measure, lightnessProperty, dataset, options?.visualMaps));
        }
    });
    // TODO
    if (options?.tooltip?.trigger === 'axis') {
        gridOptions.tooltip.push({
            trigger: 'axis'
        });
    }
    else {
        gridOptions.tooltip.push({});
    }
    // legend
    if (options?.legend) {
        gridOptions.legend.push(options.legend);
    }
    // Userdefined VisualMaps
    gridOptions.visualMap = gridOptions.visualMap.map((item, index) => {
        return {
            ...item,
            ...omitBlank(options?.visualMaps?.[index])
        };
    });
    return gridOptions;
}
export function calendarCoordinate(data, chartAnnotation, entityType, settings, options) {
    const category = getChartCategory(chartAnnotation);
    const categoryProperty = getEntityProperty(entityType, category);
    const categoryCaption = getPropertyCaption(categoryProperty);
    const measure = chartAnnotation.measures[0];
    const calendar = getChartCalendar(chartAnnotation, entityType);
    const calendarCaption = getPropertyCaption(getEntityHierarchy(entityType, calendar));
    const calendarLevelProperty = getEntityLevel(entityType, calendar);
    const formatter = calendarLevelProperty.semantics?.formatter || 'yyyyMMdd';
    const seriesComponents = measuresToSeriesComponents(chartAnnotation.measures, data, entityType, settings);
    const calendarMin = minBy(data, calendarCaption)?.[calendarCaption];
    const calendarMax = maxBy(data, calendarCaption)?.[calendarCaption];
    const dataset = {
        source: data
    };
    const gridOptions = {
        type: 'Calendar',
        name: '',
        datasets: [],
        visualMap: [],
        grid: {},
        calendar: mergeOptions({
            orient: chartAnnotation.chartType.orient === ChartOrient.vertical ? 'vertical' : 'horizontal',
            range: [
                format(parse(calendarMin, formatter, new Date()), 'yyyy-MM-dd'),
                format(parse(calendarMax, formatter, new Date()), 'yyyy-MM-dd'),
            ]
        }, options?.calendar),
        tooltip: []
    };
    gridOptions.datasets = [
        {
            dataset,
            series: seriesComponents.map((seriesComponent) => {
                const { series, visualMaps } = serializeSeriesComponent(dataset, seriesComponent, entityType, null, settings, options);
                if (seriesComponent.palette?.name) {
                    const visualMap = getMeasurePaletteVisualMap(seriesComponent.palette, seriesComponent, dataset.source);
                    gridOptions.visualMap.push(visualMap);
                }
                // series.tooltip = tooltip
                series.coordinateSystem = 'calendar';
                series.data = data.map((item) => [
                    format(parse(item[calendarCaption], formatter, new Date()), 'yyyy-MM-dd'),
                    item[measure.measure]
                ]);
                mergeOptions(series, options?.seriesStyle);
                return series;
            })
        }
    ];
    gridOptions.tooltip.push(mergeOptions({
        valueFormatter: (value) => Number(value).toFixed(2)
    }, options?.tooltip));
    // Default VisualMap
    if (isEmpty(gridOptions.visualMap)) {
        const measure = chartAnnotation.measures[0];
        const lightnessProperty = getEntityProperty(entityType, measure);
        gridOptions.visualMap.push(getColorLightnessVisualMap(measure, lightnessProperty, dataset, options?.visualMaps));
    }
    // Userdefined VisualMaps
    gridOptions.visualMap = gridOptions.visualMap.map((item, index) => {
        return {
            ...item,
            ...omitBlank(options?.visualMaps?.[index])
        };
    });
    return gridOptions;
}
export function getChartCalendar(chartAnnotation, entityType) {
    return chartAnnotation.dimensions.find((item) => {
        const property = getEntityProperty(entityType, item);
        if (property.semantics?.semantic) {
            return true;
        }
        return false;
    });
}
//# sourceMappingURL=heatmap.js.map