import { ChartOrient, displayByBehaviour, getChartCategory, getEntityHierarchy, getMemberFromRow, getPropertyHierarchy, getPropertyMeasure, hierarchize, mergeOptions } from '@metad/ocap-core';
import { TreemapChart } from 'echarts/charts';
import { use } from 'echarts/core';
import { groupBy, sumBy } from 'lodash-es';
import { DefaultValueFormatter } from './common';
use([TreemapChart]);
export function tree(data, chartAnnotation, entityType, settings, options) {
    const baseSeries = {
        type: 'tree',
        universalTransition: true,
        initialTreeDepth: 2,
        animationDurationUpdate: 750,
        emphasis: {
            focus: 'descendant'
        }
    };
    if (chartAnnotation.chartType.variant === 'radial') {
        baseSeries.layout = 'radial';
    }
    else {
        baseSeries.orient =
            chartAnnotation.chartType.orient === ChartOrient.vertical
                ? chartAnnotation.chartType.variant === 'reverse'
                    ? 'BT'
                    : 'vertical'
                : chartAnnotation.chartType.variant === 'reverse'
                    ? 'RL'
                    : 'horizontal';
    }
    if (data.schema?.recursiveHierarchy) {
        const series = mergeOptions(baseSeries, {
            data: hierarchize(data.data, data.schema?.recursiveHierarchy)
        }, options?.seriesStyle);
        const tooltip = mergeOptions({ valueFormatter: DefaultValueFormatter }, options?.tooltip);
        return {
            tooltip,
            series: [series]
        };
    }
    const { nodes, links } = leveledGraph(data.data, chartAnnotation, entityType, sumBy);
    const mainMeasureName = getPropertyMeasure(chartAnnotation.measures[0]);
    return {
        series: [
            {
                type: 'tree',
                id: mainMeasureName,
                universalTransition: true,
                data: nodes,
                links: links
            }
        ]
    };
}
export function treemap(result, chartAnnotation, entityType, settings, options) {
    const measures = chartAnnotation.measures;
    const mainMeasureName = getPropertyMeasure(measures[0]);
    let data;
    if (result.schema?.recursiveHierarchy) {
        data = hierarchize(result.data, result.schema?.recursiveHierarchy, { valueProperty: mainMeasureName });
        if (options?.seriesStyle?.hasAll) {
            data = data[0]?.children ?? [];
        }
    }
    else {
        data = leveledHierarchy(result.data, chartAnnotation, entityType, sumBy)?.[0]?.children ?? [];
    }
    const tooltip = mergeOptions({ valueFormatter: DefaultValueFormatter }, options?.tooltip);
    const series = mergeOptions({
        type: 'treemap',
        id: mainMeasureName,
        animationDurationUpdate: 1000,
        data,
        universalTransition: true,
        // label: {
        //   show: true
        // },
        // breadcrumb: {
        //   show: false
        // },
        // levels: [
        //   {
        //     itemStyle: {
        //       borderColor: '#555',
        //       borderWidth: 4,
        //       gapWidth: 4
        //     }
        //   },
        //   {
        //     colorSaturation: [0.3, 0.6],
        //     itemStyle: {
        //       borderColorSaturation: 0.7,
        //       gapWidth: 2,
        //       borderWidth: 2
        //     }
        //   },
        //   {
        //     colorSaturation: [0.3, 0.5],
        //     itemStyle: {
        //       borderColorSaturation: 0.6,
        //       gapWidth: 1
        //     }
        //   },
        //   {
        //     colorSaturation: [0.3, 0.5]
        //   }
        // ]
    }, options?.seriesStyle);
    return {
        tooltip,
        series: [series]
    };
}
export function sunburst(result, chartAnnotation, entityType, settings, options) {
    const measures = chartAnnotation.measures;
    const mainMeasureName = getPropertyMeasure(measures[0]);
    let data;
    if (result.schema?.recursiveHierarchy) {
        data = hierarchize(result.data, result.schema?.recursiveHierarchy, { valueProperty: mainMeasureName });
        if (options?.seriesStyle?.hasAll) {
            data = data[0]?.children ?? [];
        }
    }
    else {
        data = leveledHierarchy(result.data, chartAnnotation, entityType, sumBy)?.[0].children;
    }
    const tooltip = mergeOptions({ valueFormatter: DefaultValueFormatter }, options?.tooltip);
    const series = mergeOptions({
        type: 'sunburst',
        id: mainMeasureName,
        radius: ['20%', '90%'],
        animationDurationUpdate: 1000,
        nodeClick: undefined,
        data,
        universalTransition: true,
        itemStyle: {
            borderWidth: 1,
            borderColor: 'rgba(255,255,255,.5)'
        },
    }, options?.seriesStyle);
    return {
        tooltip,
        series: [
            series
        ]
    };
}
export function sankey(data, chartAnnotation, entityType, settings, options) {
    const category = getChartCategory(chartAnnotation);
    const mainMeasureName = getPropertyMeasure(chartAnnotation.measures[0]);
    const recursiveHierarchy = data.schema?.recursiveHierarchy;
    const { nodes, links } = recursiveHierarchy
        ? convertTree2NodeLinks(data.data, getEntityHierarchy(entityType, { hierarchy: recursiveHierarchy.valueProperty }), recursiveHierarchy.parentNodeProperty, recursiveHierarchy.levelProperty, category, mainMeasureName, options)
        : leveledGraph(data.data, chartAnnotation, entityType, sumBy);
    const tooltip = mergeOptions({
        formatter: (params) => {
            return params.dataType === 'edge'
                ? `${params.data.sourceCaption} > ${params.data.targetCaption}: ${DefaultValueFormatter(params.value)}`
                : `${params.data.caption}: ${DefaultValueFormatter(params.value)}`;
        },
        valueFormatter: DefaultValueFormatter
    }, options?.tooltip);
    return {
        tooltip,
        series: [
            mergeOptions({
                type: 'sankey',
                id: mainMeasureName,
                universalTransition: true,
                data: nodes,
                links: links,
                orient: chartAnnotation.chartType.orient === ChartOrient.vertical ? 'vertical' : 'horizontal'
            }, options?.seriesStyle)
        ]
    };
}
export function leveledHierarchy(data, chartAnnotation, entityType, aggregator) {
    const levels = chartAnnotation.dimensions;
    const measures = chartAnnotation.measures;
    const measureName = getPropertyMeasure(measures[0]);
    const results = [
        {
            name: 'all',
            children: data,
            value: null
        }
    ];
    let result = results;
    levels.forEach((level) => {
        const levelName = getPropertyHierarchy(level);
        const levelProperty = getEntityHierarchy(entityType, level);
        const _result = result;
        result = [];
        _result.forEach((parent) => {
            const childs = groupBy(parent.children, levelName);
            parent.children = Object.keys(childs).map((key) => {
                const item = childs[key][0];
                const caption = displayByBehaviour(getMemberFromRow(item, levelProperty), level.displayBehaviour);
                const child = {
                    key,
                    id: key,
                    name: caption,
                    children: childs[key],
                    value: aggregator(childs[key], measureName)
                };
                result.push(child);
                return child;
            });
        });
    });
    return results;
}
/**
 *
 * @param data
 * @param chartAnnotation
 * @param aggregator
 */
export function leveledGraph(data, chartAnnotation, entityType, aggregator) {
    const levels = chartAnnotation.dimensions;
    const measures = chartAnnotation.measures;
    const measureName = getPropertyMeasure(measures[0]);
    let result = [
        {
            name: 'all',
            caption: 'All',
            children: data
        }
    ];
    const nodes = {};
    const links = [];
    levels.forEach((level) => {
        const levelName = getPropertyHierarchy(level);
        const levelProperty = getEntityHierarchy(entityType, level);
        const _result = result;
        result = [];
        _result.forEach((parent) => {
            const childs = groupBy(parent.children, levelName);
            Object.keys(childs).forEach((key) => {
                const item = childs[key][0];
                const caption = displayByBehaviour(getMemberFromRow(item, levelProperty), level.displayBehaviour);
                if (!nodes[key]) {
                    nodes[key] = {
                        name: key,
                        caption,
                        label: {
                            formatter: caption
                        }
                    };
                }
                const child = {
                    name: key,
                    caption,
                    children: childs[key],
                    parent: parent.name ? parent : null
                };
                result.push(child);
                if (parent.name) {
                    links.push({
                        source: parent.name,
                        sourceCaption: parent.caption,
                        target: key,
                        targetCaption: child.caption,
                        value: aggregator(childs[key], measureName)
                    });
                }
            });
        });
    });
    return { nodes: Object.values(nodes), links };
}
export function convertTree2NodeLinks(data, child, parent, level, dimension, measure, options) {
    const nodes = [];
    const links = [];
    // let sum = 0
    const totalPlaceholder = 'Total';
    // 解析nodes 和 links
    const nodesMap = new Map();
    data.forEach((item) => {
        const caption = displayByBehaviour(getMemberFromRow(item, child), dimension.displayBehaviour);
        nodesMap.set(item[child.name], {
            name: item[child.name],
            caption,
            depth: level ? item[level] : null,
            value: Math.abs(item[measure]),
            rawValue: item,
            label: {
                formatter: caption
            }
        });
        links.push({
            source: item[parent] || totalPlaceholder,
            target: item[child.name],
            targetCaption: caption,
            value: Math.abs(item[measure]),
            rawValue: item,
            label: {
                formatter: caption
            }
        });
    });
    let totalValue = 0;
    links.forEach((x) => {
        if (x.source === totalPlaceholder) {
            totalValue += Number(x.value);
        }
    });
    if (!options?.seriesStyle?.hasAll) {
        nodes.push({
            name: totalPlaceholder,
            depth: 0,
            value: totalValue
        });
    }
    nodesMap.forEach((node) => {
        nodes.push(node);
    });
    nodes.forEach((x) => {
        x.totalValue = totalValue;
    });
    links.forEach((x) => {
        const source = nodes.find((node) => node.name === x.source);
        x.totalValue = totalValue;
        x.sourceValue = source?.value;
        x.sourceCaption = source?.caption;
    });
    return {
        nodes,
        links
    };
}
//# sourceMappingURL=hierarchy.js.map